@@ -38,7 +38,10 @@
 /* .PP
 /*	The \fIkey\fR and \fIvalue\fR are processed as is, except that
 /*	surrounding white space is stripped off. Whitespace in lookup
-/*	keys is supported as of Postfix 3.2.
+/*	keys is supported in Postfix 3.2 and later, by surrounding the
+/*	key with double quote characters `"'. Within the double quotes,
+/*	double quote `"' and backslash `\\' characters can be included
+/*	by quoting them with a preceding backslash.
 /*
 /*	When the \fB-F\fR option is given, the \fIvalue\fR must
 /*	specify one or more filenames separated by comma and/or
@@ -78,8 +81,8 @@
 /*	headers and for attached message/* headers.
 /* .sp
 /*	NOTE: with "smtputf8_enable = yes", the \fB-b\fR option
-/*	option disables UTF-8 syntax checks on query keys and
-/*	lookup results. Specify the \fB-U\fR option to force UTF-8
+/*	disables UTF-8 syntax checks on query keys and lookup
+/*	results. Specify the \fB-U\fR option to force UTF-8
 /*	syntax checks anyway.
 /* .sp
 /*	This feature is available in Postfix version 2.6 and later.
@@ -271,12 +274,12 @@
 /*	The default database type for use in \fBnewaliases\fR(1), \fBpostalias\fR(1)
 /*	and \fBpostmap\fR(1) commands.
 /* .IP "\fBimport_environment (see 'postconf -d' output)\fR"
-/*	The list of environment parameters that a privileged Postfix
+/*	The list of environment variables that a privileged Postfix
 /*	process will import from a non-Postfix parent process, or name=value
 /*	environment overrides.
 /* .IP "\fBsmtputf8_enable (yes)\fR"
 /*	Enable preliminary SMTPUTF8 support for the protocols described
-/*	in RFC 6531..6533.
+/*	in RFC 6531, RFC 6532, and RFC 6533.
 /* .IP "\fBsyslog_facility (mail)\fR"
 /*	The syslog facility of Postfix logging.
 /* .IP "\fBsyslog_name (see 'postconf -d' output)\fR"
@@ -340,6 +343,7 @@
 #include <set_eugid.h>
 #include <warn_stat.h>
 #include <clean_env.h>
+#include <dict_db.h>
 
 /* Global library. */
 
@@ -347,7 +351,6 @@
 #include <mail_dict.h>
 #include <mail_params.h>
 #include <mail_version.h>
-#include <mkmap.h>
 #include <mail_task.h>
 #include <dict_proxy.h>
 #include <mime_state.h>
@@ -432,6 +435,24 @@ static void postmap(char *map_type, char *path_name, int postmap_flags,
 	&& (st.st_uid != geteuid() || st.st_gid != getegid()))
 	set_eugid(st.st_uid, st.st_gid);
 
+    /*
+     * Override the default per-table cache size for DB map (re)builds. We
+     * can't do this in the mkmap* functions because those don't have access
+     * to Postfix parameter settings.
+     * 
+     * db_cache_size" is defined in util/dict_open.c and defaults to 128kB,
+     * which works well for the lookup code.
+     * 
+     * We use a larger per-table cache when building ".db" files. For "hash"
+     * files performance degrades rapidly unless the memory pool is O(file
+     * size).
+     * 
+     * For "btree" files performance is good with sorted input even for small
+     * memory pools, but with random input degrades rapidly unless the memory
+     * pool is O(file size).
+     */
+    dict_db_cache_size = var_db_create_buf;
+
     /*
      * Open the database, optionally create it when it does not exist,
      * optionally truncate it when it does exist, and lock out any
@@ -657,8 +678,8 @@ static int postmap_queries(VSTREAM *in, char **maps, const int map_count,
 	dicts[n] = 0;
 
     /*
-     * Perform all queries. Open maps on the fly, to avoid opening unecessary
-     * maps.
+     * Perform all queries. Open maps on the fly, to avoid opening
+     * unnecessary maps.
      */
     if ((postmap_flags & POSTMAP_FLAG_HB_KEY) == 0) {
 	while (vstring_get_nonl(keybuf, in) != VSTREAM_EOF) {
