@@ -172,8 +172,10 @@
 /*	on the same MTA. Larger sites would have to share the
 /*	\fBpostscreen\fR(8) cache between primary and backup MTAs,
 /*	which would introduce a common point of failure.
-/* .IP "\fBpostscreen_whitelist_interfaces (static:all)\fR"
-/*	Renamed to postscreen_allowlist_interfaces in Postfix 3.6.
+/* .IP "\fBpostscreen_allowlist_interfaces (static:all)\fR"
+/*	A list of local \fBpostscreen\fR(8) server IP addresses where a
+/*	non-allowlisted remote SMTP client can obtain \fBpostscreen\fR(8)'s temporary
+/*	allowlist status.
 /* BEFORE 220 GREETING TESTS
 /* .ad
 /* .fi
@@ -190,11 +192,12 @@
 /*	with the postscreen_dnsbl_sites and postscreen_dnsbl_threshold
 /*	parameters).
 /* .IP "\fBpostscreen_dnsbl_reply_map (empty)\fR"
-/*	A mapping from actual DNSBL domain name which includes a secret
+/*	A mapping from an actual DNSBL domain name which includes a secret
 /*	password, to the DNSBL domain name that postscreen will reply with
 /*	when it rejects mail.
 /* .IP "\fBpostscreen_dnsbl_sites (empty)\fR"
-/*	Optional list of DNS allow/denylist domains, filters and weight
+/*	Optional list of patterns with DNS allow/denylist domains, filters
+/*	and weight
 /*	factors.
 /* .IP "\fBpostscreen_dnsbl_threshold (1)\fR"
 /*	The inclusive lower bound for blocking a remote SMTP client, based on
@@ -989,7 +992,7 @@ static void pre_accept(char *unused_name, char **unused_argv)
     if (new_event_time >= last_event_time + 1
 	&& (name = dict_changed_name()) != 0) {
 	msg_info("table %s has changed - finishing in the background", name);
-	psc_drain(unused_name, unused_argv);
+	event_server_drain();
     } else {
 	last_event_time = new_event_time;
     }
