@@ -81,7 +81,7 @@
 /*	command above.
 /* .IP \fB-bl\fR
 /*	Go into daemon mode. To accept only local connections as
-/*	with Sendmail\'s \fB-bl\fR option, specify "\fBinet_interfaces
+/*	with Sendmail's \fB-bl\fR option, specify "\fBinet_interfaces
 /*	= loopback\fR" in the Postfix \fBmain.cf\fR configuration
 /*	file.
 /* .IP \fB-bm\fR
@@ -140,7 +140,7 @@
 /*	Initialize alias database. See the \fBnewaliases\fR
 /*	command above.
 /* .IP "\fB-i\fR"
-/*	When reading a message from standard input, don\'t treat a line
+/*	When reading a message from standard input, don't treat a line
 /*	with only a \fB.\fR character as the end of input.
 /* .IP "\fB-L \fIlabel\fR (ignored)"
 /*	The logging label. Use the \fBsyslog_name\fR configuration
@@ -170,7 +170,7 @@
 /*	To send 8-bit or binary content, use an appropriate MIME encapsulation
 /*	and specify the appropriate \fB-B\fR command-line option.
 /* .IP "\fB-oi\fR"
-/*	When reading a message from standard input, don\'t treat a line
+/*	When reading a message from standard input, don't treat a line
 /*	with only a \fB.\fR character as the end of input.
 /* .IP "\fB-om\fR (ignored)"
 /*	The sender is never eliminated from alias etc. expansions.
@@ -403,7 +403,7 @@
 /*	The time after which the sender receives a copy of the message
 /*	headers of mail that is still queued.
 /* .IP "\fBimport_environment (see 'postconf -d' output)\fR"
-/*	The list of environment parameters that a privileged Postfix
+/*	The list of environment variables that a privileged Postfix
 /*	process will import from a non-Postfix parent process, or name=value
 /*	environment overrides.
 /* .IP "\fBmail_owner (postfix)\fR"
@@ -694,6 +694,7 @@ static void enqueue(const int flags, const char *encoding,
     VSTRING *postdrop_command;
     uid_t   uid = getuid();
     int     status;
+    VSTRING *why;			/* postdrop status message */
     int     naddr;
     int     prev_type;
     MIME_STATE *mime_state = 0;
@@ -987,11 +988,15 @@ static void enqueue(const int flags, const char *encoding,
     if (vstream_ferror(VSTREAM_IN))
 	msg_fatal_status(EX_DATAERR, "%s(%ld): error reading input: %m",
 			 saved_sender, (long) uid);
-    if ((status = mail_stream_finish(handle, (VSTRING *) 0)) != 0)
+    why = vstring_alloc(100);
+    if ((status = mail_stream_finish(handle, why)) != CLEANUP_STAT_OK)
 	msg_fatal_status((status & CLEANUP_STAT_BAD) ? EX_SOFTWARE :
 			 (status & CLEANUP_STAT_WRITE) ? EX_TEMPFAIL :
+			 (status & CLEANUP_STAT_NOPERM) ? EX_NOPERM :
 			 EX_UNAVAILABLE, "%s(%ld): %s", saved_sender,
-			 (long) uid, cleanup_strerror(status));
+			 (long) uid, VSTRING_LEN(why) ?
+			 STR(why) : cleanup_strerror(status));
+    vstring_free(why);
 
     /*
      * Don't leave them in the dark.
