@@ -139,6 +139,7 @@
 #include <sock_addr.h>
 #include <inet_proto.h>
 #include <split_at.h>
+#include <inet_prefix_top.h>
 
 /* Global library. */
 
@@ -216,13 +217,8 @@ static int smtpd_peer_sockaddr_to_hostaddr(SMTPD_STATE *state)
 	 */
 	if ((aierr = sockaddr_to_hostaddr(sa, sa_length, &client_addr,
 					  &client_port, 0)) != 0)
-	    msg_fatal("%s: cannot convert client sockaddr type %s length %ld "
-		      "to string: %s", myname,
-#ifdef AF_INET6
-		      sa->sa_family == AF_INET6 ? "AF_INET6" :
-#endif
-		      sa->sa_family == AF_INET ? "AF_INET" : "other",
-		      (long) sa_length, MAI_STRERROR(aierr));
+	    msg_fatal("%s: cannot convert client address/port to string: %s",
+		      myname, MAI_STRERROR(aierr));
 	state->port = mystrdup(client_port.buf);
 
 	/*
@@ -255,10 +251,12 @@ static int smtpd_peer_sockaddr_to_hostaddr(SMTPD_STATE *state)
 		state->addr = mystrdup(colonp + 1);
 		state->rfc_addr = mystrdup(colonp + 1);
 		state->addr_family = AF_INET;
-		aierr = hostaddr_to_sockaddr(state->addr, (char *) 0, 0, &res0);
+		aierr =
+		    hostaddr_to_sockaddr(state->addr, state->port, 0, &res0);
 		if (aierr)
-		    msg_fatal("%s: cannot convert %s from string to binary: %s",
-			      myname, state->addr, MAI_STRERROR(aierr));
+		    msg_fatal("%s: cannot convert [%s]:%s to binary: %s",
+			      myname, state->addr, state->port,
+			      MAI_STRERROR(aierr));
 		sa_length = res0->ai_addrlen;
 		if (sa_length > sizeof(state->sockaddr))
 		    sa_length = sizeof(state->sockaddr);
@@ -269,10 +267,10 @@ static int smtpd_peer_sockaddr_to_hostaddr(SMTPD_STATE *state)
 	    /*
 	     * Following RFC 2821 section 4.1.3, an IPv6 address literal gets
 	     * a prefix of 'IPv6:'. We do this consistently for all IPv6
-	     * addresses that appear in headers or envelopes. The fact
-	     * that valid_mailhost_addr() enforces the form helps of course.
-	     * We use the form without IPV6: prefix when doing access
-	     * control, or when accessing the connection cache.
+	     * addresses that appear in headers or envelopes. The fact that
+	     * valid_mailhost_addr() enforces the form helps of course. We
+	     * use the form without IPV6: prefix when doing access control,
+	     * or when accessing the connection cache.
 	     */
 	    else {
 		state->addr = mystrdup(client_addr.buf);
@@ -301,15 +299,9 @@ static int smtpd_peer_sockaddr_to_hostaddr(SMTPD_STATE *state)
 					  state->dest_sockaddr_len,
 					  &server_addr,
 					  &server_port, 0)) != 0)
-	    /* TODO: convert IPv4-in-IPv6 to IPv4 form. */
-	    msg_fatal("%s: cannot convert server sockaddr type %s length %ld "
-		    "to string: %s", myname,
-#ifdef AF_INET6
-		   state->dest_sockaddr.ss_family == AF_INET6 ? "AF_INET6" :
-#endif
-		      state->dest_sockaddr.ss_family == AF_INET ? "AF_INET" :
-		      "other", (long) state->dest_sockaddr_len,
-		      MAI_STRERROR(aierr));
+	    msg_fatal("%s: cannot convert server address/port to string: %s",
+		      myname, MAI_STRERROR(aierr));
+	/* TODO: convert IPv4-in-IPv6 to IPv4 form. */
 	state->dest_addr = mystrdup(server_addr.buf);
 	state->dest_port = mystrdup(server_port.buf);
 
@@ -417,8 +409,8 @@ static void smtpd_peer_hostaddr_to_sockaddr(SMTPD_STATE *state)
 
     if ((aierr = hostaddr_to_sockaddr(state->addr, state->port,
 				      SOCK_STREAM, &res)) != 0)
-	msg_fatal("%s: cannot convert client address '%s' port '%s' to binary: %s",
-		  myname, state->addr, state->port, MAI_STRERROR(aierr));
+	msg_fatal("%s: cannot convert client address/port to string: %s",
+		  myname, MAI_STRERROR(aierr));
     if (res->ai_addrlen > sizeof(state->sockaddr))
 	msg_panic("%s: address length > struct sockaddr_storage", myname);
     memcpy((void *) &(state->sockaddr), res->ai_addr, res->ai_addrlen);
@@ -592,6 +584,7 @@ static void smtpd_peer_from_proxy(SMTPD_STATE *state)
 
 void    smtpd_peer_init(SMTPD_STATE *state)
 {
+    int     af;
 
     /*
      * Initialize.
@@ -610,6 +603,7 @@ void    smtpd_peer_init(SMTPD_STATE *state)
     state->namaddr = 0;
     state->rfc_addr = 0;
     state->port = 0;
+    state->anvil_range = 0;
     state->dest_addr = 0;
     state->dest_port = 0;
 
@@ -644,6 +638,19 @@ void    smtpd_peer_init(SMTPD_STATE *state)
      */
     state->namaddr = SMTPD_BUILD_NAMADDRPORT(state->name, state->addr,
 					     state->port);
+
+    /*
+     * Generate 'address' or 'net/mask' index for anvil event aggregation.
+     * Don't do this for non-socket input. See smtpd_peer_not_inet().
+     */
+    if (state->addr_family != AF_UNSPEC) {
+	af = SOCK_ADDR_FAMILY(&(state->sockaddr));
+	state->anvil_range = inet_prefix_top(af,
+					SOCK_ADDR_ADDRP(&(state->sockaddr)),
+					     af == AF_INET ?
+					     var_smtpd_cipv4_prefix :
+					     var_smtpd_cipv6_prefix);
+    }
 }
 
 /* smtpd_peer_reset - destroy peer information */
@@ -666,4 +673,6 @@ void    smtpd_peer_reset(SMTPD_STATE *state)
 	myfree(state->dest_addr);
     if (state->dest_port)
 	myfree(state->dest_port);
+    if (state->anvil_range)
+	myfree(state->anvil_range);
 }
